/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "oirSupport.h"
#include "fitsio.h"
#include "imaging_data.h"
#include "imaging_fdata.h"

/**********************************************************
 * Compute average  frame 
 * Usage:
 *    Average  infile outfile
 *        infile is blank separated, quoted list of a sequence
 *            of IMAGING_DATA tables
 *        outfile is IMAGING_FDATA table with result.
 ************************************************************* */

/* Local prototypes   */
void ReadData(char **fileList, int nFiles, int nRegion, 
		long *npix, float **Data, long *nOn,  int *ierr);

int main (int argc, char** argv) {

#define MAXREGION 12 /* Maximum number of regions */
   char **fileList;
   int nFiles;
   int i,ierr;
   char outFileName[130];
   char *bang="!";
   fitsfile *inFile;
   fitsfile *outFile = NULL;
   imaging_fdata_table_def *odata;
   imaging_detector_table_def *DetectorDef;
   imaging_fdata_row *outData;
   int nx, ny, nxList[MAXREGION], nyList[MAXREGION];
   int nRegion, iReg;
   long npix, iPix, nnpix[MAXREGION], nOn;
   float *Data[MAXREGION];

   ierr = 0;
/*        
       break up file list and print what you find 
       then read and unchop files
*/
   fileList = oirStrSplit(argv[1], &nFiles);

/*
      open first file
*/
   i = fits_open_file(&inFile, fileList[0], READONLY, &ierr) ;
   if (ierr != 0) {
   printf("cfitsio error %i opening file %s\n", ierr, fileList[0]);
   printf("%s \n",strerror(errno));
   return 1;
   }
   ierr = 0;
/*
      move to detector section
*/
   DetectorDef = read_imaging_detector_table(inFile, &ierr);
   if (ierr !=0) {
   printf("error %i opening detector table in %s\n", 
	  ierr, fileList[0]);
   return 1;
   }
   nRegion = DetectorDef->nregion;
   for (iReg=0;iReg<nRegion;iReg++) {
     nxList[iReg] = DetectorDef->naxis[iReg][0];
     nyList[iReg] = DetectorDef->naxis[iReg][1];
     npix = nyList[iReg] * (long)nxList[iReg];
     nnpix[iReg] = npix;
   }
   
/*
      allocate accumulator array
*/
   for (iReg=0; iReg<nRegion; iReg++) {
     npix = nnpix[iReg];
     Data[iReg] = (float *)malloc(npix * sizeof(float));
     memset (Data[iReg], 0, npix*sizeof(float));
   }
   
   i = fits_close_file(inFile, &ierr);

/* 
       read data 
*/
   ReadData(fileList, nFiles, nRegion, nnpix, Data,  &nOn, &ierr);

/* 
     give some information
*/
    fprintf(stdout, "averaged %i frames\n", nOn);
/*
      prepare output file
*/
   strcpy(outFileName, bang);
   strcat(outFileName, argv[2]);

   fits_create_file (&outFile, outFileName, &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error opening fits output %s %i\n",argv[2], ierr);
      printf("%s \n",strerror(errno));
      return 1;
   }

   odata = make_imaging_fdata_table_def (outFile, DetectorDef, 1, &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error creating data definition\n");
      return 1;
   }

/*
     create and initialize row 
*/
   outData = make_imaging_fdata_row(odata);
   outData->frame = 1;
   for (iReg=0; iReg<nRegion; iReg++) {
     outData->targtype[iReg] = 'S';
   }

/* 
      copy to output row 
*/
   for (iReg=0; iReg<nRegion; iReg++) {
     npix = nnpix[iReg];
     for (iPix=0;iPix<npix;iPix++) outData->data[iReg][iPix] = Data[iReg][iPix];
   } /* end region loop */

/*   
     Write output and close
*/
   i = write_imaging_fdata_table(outFile, odata, outData, 1, &ierr);
   i = close_imaging_fdata_table(outFile, odata, &ierr);
/*
       Cleanup
*/
   kill_imaging_fdata_row(outData);

   for (iReg=0; iReg<nRegion; iReg++) {
      if (Data[iReg])  free(Data[iReg]);
   }
   return 0;
} /* end main */

/***************************************************************
 * read through all raw files reading frames  and accumulating 
 * them in given arrays; divide by number of frames
 * Inputs:
 *    char **string     array of file names
 *    int     nFiles    number of file names
 *    int     nRegion   number of regions
 *    int    *npix      number of pixels in each region
 *    float **Data      Array of arrays for each region averaged values
 *    long    nOn       number of target frames
 *    long    ierr      error code, 0=>OK
 **************************************************************** */

void ReadData(char **fileList, int nFiles, int nRegion, 
		long *npix, float **Data, long *nOn,  int *ierr)

{

   fitsfile *inFile;
   imaging_data_table_def *idata;
   imaging_data_row *inData;
   int iFile;

   long iRow, nRow;
   int i, iReg;
   int TN, SN;
   long iPix;

   float normTarget;

   *ierr    = 0;
   TN = 0;

/* 
   Initialize output arrays 
*/
   for (iReg=0; iReg<nRegion; iReg++) {
     for (iPix=0;iPix<npix[iReg];iPix++) {
       Data[iReg][iPix] = 0.0;
     }
   }
/*
      loop over files, read rows, read , add data
      into appropriate accumulators 
*/

   for (iFile =0; iFile <nFiles; iFile++) {
/*
      open file, open table, create table row
*/
      i = fits_open_file(&inFile, fileList[iFile], READONLY, ierr);
      if (*ierr !=0) {
         printf("error %i opening file %s\n", *ierr, fileList[iFile]);
         printf("%s \n",strerror(errno));
         return;
      }
      idata = open_imaging_data_table(inFile, ierr);
      if (*ierr !=0) {
         printf("error %i opening imaging data in file %s\n", 
            *ierr, fileList[iFile]);
         return;
      }
      inData = make_imaging_data_row(idata);
      i = fits_get_num_rows (inFile,  &nRow, ierr);
      /* suppress autoscaling to avoid overflow */
      for (i=0; i<idata->nregion; i++)
         fits_set_tscale(inFile, 1+idata->data_col[i], 1.0, 0.0, ierr);

/*
         read lines
*/
      for (iRow =1; iRow<=nRow; iRow++) {
         i = read_imaging_data_table(inFile, idata, inData, iRow, ierr);
         if (*ierr !=0) {
            printf("error %i reading imaging data line %i in file %s\n", 
               *ierr, iRow, fileList[iFile]);
            return;
         }
	 /* Accumulate data */
	 for (iReg=0; iReg<nRegion; iReg++) {
  	   for (iPix=0;iPix<npix[iReg];iPix++) {
	     Data[iReg][iPix] += 32768. + (short)inData->data[iReg][iPix];
	   }
	 }
            ++TN;
      } /* read rows in one file */
      
/*
         close file and table, destroy row
*/
      i= close_imaging_data_table(inFile, idata, ierr);
      kill_imaging_data_row(inData);
   } /* end read all files in list */
/* 
   Normalize output arrays 
*/
   if (TN>0) normTarget = 1.0 / (float)TN;
   else normTarget = 1.0;
   for (iReg=0; iReg<nRegion; iReg++) {
     for (iPix=0;iPix<npix[iReg];iPix++) {
       Data[iReg][iPix]  *= normTarget;
     }
   }

   *nOn  = TN;

   return;
 } /* end of ReadData */
